@@ -5,6 +5,7 @@
 {-# LANGUAGE BangPatterns #-}
 module Network.HTTP.Conduit.Manager
     ( Manager
+    , mResponseTimeout
     , ManagerSettings (..)
     , ConnKey (..)
     , ConnHost (..)
@@ -64,6 +65,7 @@ import Network.Socks5 (SocksConf)
 import Data.Default
 import Data.Maybe (mapMaybe)
 import System.IO (Handle)
+import System.Mem.Weak (Weak, deRefWeak)
 import Data.Conduit (($$), yield, runException)
 
 -- | Settings for a @Manager@. Please use the 'def' function and then modify
@@ -75,6 +77,13 @@ data ManagerSettings = ManagerSettings
       -- ^ Check if the server certificate is valid. Only relevant for HTTPS.
     , managerCertStore :: IO CertificateStore
       -- ^ Load up the certificate store. By default uses the system store.
+    , managerResponseTimeout :: Maybe Int
+      -- ^ Default timeout (in microseconds) to be applied to requests which do
+      -- not provide a timeout value.
+      --
+      -- Default is 5 seconds
+      --
+      -- Since 1.9.3
     }
 
 type X509Encoded = L.ByteString
@@ -84,6 +93,7 @@ instance Default ManagerSettings where
         { managerConnCount = 10
         , managerCheckCerts = defaultCheckCerts
         , managerCertStore = getSystemCertificateStore
+        , managerResponseTimeout = Just 5000000
         }
 
 -- | Check certificates using the operating system's certificate checker.
@@ -105,6 +115,8 @@ data Manager = Manager
     , mCertCache :: !(I.IORef (Map.Map S8.ByteString (Map.Map X509Encoded UTCTime)))
     -- ^ Cache of validated certificates. The @UTCTime@ gives the expiration
     -- time for the validity of the certificate. The @Ascii@ is the hostname.
+    , mResponseTimeout :: !(Maybe Int)
+    -- ^ Copied from 'managerResponseTimeout'
     }
 
 data NonEmptyList a =
@@ -172,19 +184,33 @@ newManager ms = do
                     return certStore
                 Just x -> return x
     mapRef <- I.newIORef (Just Map.empty)
+    wmapRef <- I.mkWeakIORef mapRef $ closeManager' mapRef
     certCache <- I.newIORef Map.empty
-    _ <- forkIO $ reap mapRef certCache
-    return $ Manager mapRef (managerConnCount ms) (\x y -> getCertStore >>= \cs -> managerCheckCerts ms cs x y) certCache
+    _ <- forkIO $ reap wmapRef certCache
+    let manager = Manager
+            { mConns = mapRef
+            , mMaxConns = managerConnCount ms
+            , mCheckCerts = \x y -> getCertStore >>= \cs -> managerCheckCerts ms cs x y
+            , mCertCache = certCache
+            , mResponseTimeout = managerResponseTimeout ms
+            }
+    return manager
 
 -- | Collect and destroy any stale connections.
-reap :: I.IORef (Maybe (Map.Map ConnKey (NonEmptyList ConnInfo)))
+reap :: Weak (I.IORef (Maybe (Map.Map ConnKey (NonEmptyList ConnInfo))))
      -> I.IORef (Map.Map S8.ByteString (Map.Map X509Encoded UTCTime))
      -> IO ()
-reap mapRef certCacheRef =
+reap wmapRef certCacheRef =
     mask_ loop
   where
     loop = do
         threadDelay (5 * 1000 * 1000)
+        mmapRef <- deRefWeak wmapRef
+        case mmapRef of
+            Nothing -> return () -- manager is closed
+            Just mapRef -> goMapRef mapRef
+
+    goMapRef mapRef = do
         now <- getCurrentTime
         let isNotStale time = 30 `addUTCTime` time >= now
         mtoDestroy <- I.atomicModifyIORef mapRef (findStaleWrap isNotStale)
@@ -286,11 +312,19 @@ withManagerSettings s f = runResourceT $ do
     (_, manager) <- allocate (newManager s) closeManager
     f manager
 
--- | Close all connections in a 'Manager'. Afterwards, the
--- 'Manager' can be reused if desired.
+-- | Close all connections in a 'Manager'. Afterwards, the 'Manager'
+-- can be reused if desired.
+--
+-- Note that this doesn't affect currently in-flight connections,
+-- meaning you can safely use it without hurting any queries you may
+-- have concurrently running.
 closeManager :: Manager -> IO ()
-closeManager manager = mask_ $ do
-    m <- I.atomicModifyIORef (mConns manager) $ \x -> (Nothing, x)
+closeManager = closeManager' . mConns
+
+closeManager' :: I.IORef (Maybe (Map.Map ConnKey (NonEmptyList ConnInfo)))
+              -> IO ()
+closeManager' connsRef = mask_ $ do
+    m <- I.atomicModifyIORef connsRef $ \x -> (Nothing, x)
     mapM_ (nonEmptyMapM_ safeConnClose) $ maybe [] Map.elems m
 
 safeConnClose :: ConnInfo -> IO ()
