@@ -16,6 +16,7 @@ module Network.HTTP.Conduit.Types
     ) where
 
 import Data.Int (Int64)
+import Data.Word (Word64)
 import Data.Typeable (Typeable)
 
 import Blaze.ByteString.Builder
@@ -191,6 +192,16 @@ data HttpException = StatusCodeException W.Status W.ResponseHeaders CookieJar
                    | InvalidHeader S.ByteString
                    | InternalIOException IOException
                    | ProxyConnectException S.ByteString Int (Either S.ByteString HttpException) -- ^ host/port
+                   | NoResponseDataReceived
+                   | TlsException SomeException
+                   | ResponseBodyTooShort Word64 Word64
+                   -- ^ Expected size/actual size.
+                   --
+                   -- Since 1.9.4
+                   | InvalidChunkHeaders
+                   -- ^
+                   --
+                   -- Since 1.9.4
     deriving (Show, Typeable)
 instance Exception HttpException
 
@@ -225,10 +236,10 @@ data Cookie = Cookie
   , cookie_secure_only :: Bool
   , cookie_http_only :: Bool
   }
-  deriving (Show)
+  deriving (Read, Show)
 
 newtype CookieJar = CJ { expose :: [Cookie] }
-  deriving (Show)
+  deriving (Read, Show)
 
 -- This corresponds to step 11 of the algorithm described in Section 5.3 \"Storage Model\"
 instance Eq Cookie where
